# Типы экспедиций
EXPEDITIONTYPES = { # name, description
	1: [_("expeditiontypes.cartography.name"), # Картографическая
		_("expeditiontypes.cartography.description") # "Если в конце дня у Вас осталось больше двух пунктов движения, Вы можете проложить тропу из текущего участка в соседний (это можно делать не больше одного раза на каждом участке)"
		],
	2: [_("expeditiontypes.botany.name"), # Ботаническая
		_("expeditiontypes.botany.description") # "Вам не страшна лихорадка. На этапе охоты результат броска костей увеличивается на 1"
		],
	3: [_("expeditiontypes.military.name"), # Военная
		_("expeditiontypes.military.description") # "При использовании снаряжения результат броска костей увеличивается на 2"
		], 
	4: [_("expeditiontypes.archeology.name"), # Археологическая
		_("expeditiontypes.archeology.description") # "Если Вы используете тропу для продвижения на участок, местность в котором отличается от местности нынешнего участка, то всегда обнаруживаете достопримечательность"
		], 
	5: [_("expeditiontypes.religious.name"), # Религиозная
		_("expeditiontypes.religious.description") # "Все обнаруженные поселения туземцев оказываются дружественными (но не империи)"
		], 
	6: [_("expeditiontypes.doctor.name"), # Врачебная
		_("expeditiontypes.doctor.description") # "Для избавления от лихорадки достаточно 2 джокера (вместо обычных 3). При выпадении значка смерти Вы теряете 1 еду вместо конкистадора (если еды нет - теряете конкистадора)"
		]
	}

LANDS = {
	2 : _("lands.swamp"), # Трясины
	3 : _("lands.hills"), # Холмы
	4 : _("lands.mountains"), # Горы
	5 : _("lands.jungle"), # Джунгли
	10: _("lands.forest"), # Лес
	11: _("lands.lake"), # Озеро
	12: _("lands.plains") # Равнины
}

DIRS = { # offset (x, y, z):[0], moralePoints:[1], menuItem: [2], icon: [3]
	"0" : [( 0,  0,  0), -1, _("dirs.nomove"), u'\u21B9'], # На месте
	"N" : [( 0,  1, -1), -1, _("dirs.nord"), u'\u2191'], # Север
	"NE": [( 1,  0, -1),  1, _("dirs.nord-east"), u'\u2197'], # Северо-восток
	"SE": [( 1, -1,  0),  3, _("dirs.south-east"), u'\u2198'], # Юго-восток
	"S" : [( 0, -1,  1),  1, _("dirs.south"), u'\u2193'], # Юг
	"SW": [(-1,  0,  1), -1, _("dirs.south-west"), u'\u2199'], # Юго-запад
	"NW": [(-1,  1,  0), -3, _("dirs.nord-west"), u'\u2196'] # Северо-запад
}

DIRS_ALIAS = {
	"NWN": "NNW", "NNW": "NNW", "NNE": "NNE", "NEN": "NNE",
	"SENE": "E", "NESE": "E", "SWNW": "W", "NWSW": "W",
	"SSW": "SSW", "SWS": "SSW", "SSE": "SSE", "SES": "SSE"
}

DIRS_EXTENSION = {
	"NNE": [tuple(),  0, _("dirs_extesion.nord-nord-east"), u'\u2191\u2197'], # Северо-северо-восток
	"E"  : [tuple(),  2, _("dirs_extesion.east"), u'\u2192'], # Восток
	"SSE": [tuple(),  2, _("dirs_extesion.south-south-east"), u'\u2193\u2198'], # Юго-юго-восток
	"SSW": [tuple(),  0, _("dirs_extesion.south-south-west"), u'\u2193\u2199'], # Юго-юго-запад
	"W"  : [tuple(), -2, _("dirs_extesion.west"), u'\u2194'], # Запад
	"NNW": [tuple(), -2, _("dirs_extesion.nord-nord-west"), u'\u2191\u2196'] # Северо-северо-запад
}
DIRS_EXTENSION.update(DIRS)

CONTACT_RESULTS = {
	1: _("Империя воинственных туземцев"),
	2: _("Лихорадка"),
	3: _("Поселение туземцев"),
	4: _("Тропа"),
	5: _("Еда и дружественное поселение"),
	6: _("Ящик с припасами и дружественное поселение")
}

DICE_RESULTS = {
	# Phase number: {dices: [consequences, description, callable]}
	2: {
		2 : [_("Потеря конкистадора"), _("Родригес не вернулся в лагерь")],
		3 : [_("Потеря конкистадора"), _("Санчо не проснулся")],
		4 : [_("Пункт движения и лихорадка"), "2-4"],
		5 : [_("Пункт движения и лихорадка"), "2-5"],
		6 : [_("Пункт движения и упадок духа"), "2-6"],
		7 : [_("Пункт движения и упадок духа"), "2-7"],
		8 : [_("Пункт движения и упадок духа"), "2-8"],
		9 : [_("Пункт движения"), "2-9"],
		10: [_("Два пункта движения"), "2-10"],
		11: [_("Три пункта движения"), "2-11"],
		12: [_("Четыре пункта движения"), "2-12"]
	},
	4: {
		2 : [_("Потеря конкистадора"), "4-2"],
		3 : [_("Лихорадка"), "4-3"],
		4 : [_("Потеря пункта движения"), "4-4"],
		5 : [_("Потеря пункта движения"), "4-5"],
		6 : [_("Поселение туземцев"), "4-6"],
		7 : [_("Поселение туземцев"), "4-7"],
		8 : [_("Подъём духа"), "4-8"],
		9 : [_("Тропа"), "4-9"],
		10: [_("Достопримечательность"), "4-10"],
		11: [_("Достопримечательность"), "4-11"],
		12: [_("Достопримечательность"), "4-12"]
	},
	5: {
		2 : [CONTACT_RESULTS[1], "5-2"],
		3 : [CONTACT_RESULTS[1], "5-3"],
		4 : [CONTACT_RESULTS[1], "5-4"],
		5 : [CONTACT_RESULTS[2], "5-5"],
		6 : [CONTACT_RESULTS[3], "5-6"],
		7 : [CONTACT_RESULTS[3], "5-7"],
		8 : [CONTACT_RESULTS[3], "5-8"],
		9 : [CONTACT_RESULTS[4], "5-9"],
		10: [CONTACT_RESULTS[5], "5-10"],
		11: [CONTACT_RESULTS[6], "5-11"],
		12: [CONTACT_RESULTS[6], "5-12"]
	},
	6 : {
		2 : [_("Потеря конкистадора"), "6-2"],
		3 : [_("Потеря конкистадора"), "6-3"],
		4 : [_("Упадок духа"), "6-4"],
		5 : [_("Еда и упадок духа"), "6-5"],
		6 : [_("Еда"), "6-6"],
		7 : [_("Еда"), "6-7"],
		8 : [_("Еда"), "6-8"],
		9 : [_("Еда на два дня"), "6-9"],
		10: [_("Еда на два дня"), "6-10"],
		11: [_("Еда на два дня и подъём духа"), "6-11"],
		12: [_("Еда на два дня и подъём духа"), "6-12"]
	}
}

